'Scrypt for Python'

__version__ = '1.0.2'

# First, try loading libscrypt
_done = False
try:
    from pylibscrypt import *
except ImportError:
    pass
else:
    _done = True

# If that didn't work, try the scrypt module
if not _done:
    try:
        from pyscrypt import *
    except ImportError:
        pass
    else:
        _done = True

# If that didn't work either, the inlined Python version
if not _done:
    try:
        from pypyscrypt_inline import *
    except ImportError:
        pass
    else:
        _done = True

# Finally the non-inlined
if not _done:
    from pypyscrypt import *

__all__ = ['scrypt', 'scrypt_mcf', 'scrypt_mcf_check']

# Clean up pydoc output
del __path__
del consts
del pylibscrypt




#!/usr/bin/env python

# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Scrypt implementation that calls into the 'scrypt' python module"""


from scrypt import hash as _scrypt


import mcf as mcf_mod

from consts import *


# scrypt < 0.6 doesn't support hash length
try:
    _scrypt(b'password', b'NaCl', N=2, r=1, p=1, buflen=42)
except TypeError:
    raise ImportError('scrypt module version unsupported, 0.6+ required')


def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
    """Derives a 64-byte hash using the scrypt key-derivarion function

    N must be a power of two larger than 1 but no larger than 2 ** 63 (insane)
    r and p must be positive numbers such that r * p < 2 ** 30

    The default values are:
    N -- 2**14 (~16k)
    r -- 8
    p -- 1

    Memory usage is proportional to N*r. Defaults require about 16 MiB.
    Time taken is proportional to N*p. Defaults take <100ms of a recent x86.

    The last one differs from libscrypt defaults, but matches the 'interactive'
    work factor from the original paper. For long term storage where runtime of
    key derivation is not a problem, you could use 16 as in libscrypt or better
    yet increase N if memory is plentiful.
    """
    if not isinstance(password, bytes):
        raise TypeError('scrypt password must be a byte string')
    if not isinstance(salt, bytes):
        raise TypeError('scrypt salt must be a byte string')
    if N > 2**63:
        raise ValueError('N value cannot be larger than 2**63')
    try:
        return _scrypt(password=password, salt=salt, N=N, r=r, p=p, buflen=olen)
    except:
        raise ValueError


def scrypt_mcf(password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p):
    """Derives a Modular Crypt Format hash using the scrypt KDF

    Parameter space is smaller than for scrypt():
    N must be a power of two larger than 1 but no larger than 2 ** 31
    r and p must be positive numbers between 1 and 255
    Salt must be a byte string 1-16 bytes long.

    If no salt is given, 16 random bytes are generated using os.urandom.
    """
    return mcf_mod.scrypt_mcf(scrypt, password, salt, N, r, p)


def scrypt_mcf_check(mcf, password):
    """Returns True if the password matches the given MCF hash"""
    return mcf_mod.scrypt_mcf_check(scrypt, mcf, password)


if __name__ == "__main__":
    import tests
    print('Testing scrypt...')
    tests.run_tests(scrypt, scrypt_mcf, scrypt_mcf_check)




#!/usr/bin/env python

# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Modular Crypt Format support for scrypt, compatible with libscrypt"""


import base64
import os
import struct


from consts import *


def scrypt_mcf(scrypt, password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p):
    """Derives a Modular Crypt Format hash using the scrypt KDF given

    Expects the signature:
    scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64)

    If no salt is given, 16 random bytes are generated using os.urandom."""
    if salt is None:
        salt = os.urandom(16)
    elif not (1 <= len(salt) <= 16):
        raise ValueError('salt must be 1-16 bytes')
    if r > 255:
        raise ValueError('scrypt_mcf r out of range [1,255]')
    if p > 255:
        raise ValueError('scrypt_mcf p out of range [1,255]')
    if N > 2**31:
        raise ValueError('scrypt_mcf N out of range [2,2**31]')

    hash = scrypt(password, salt, N, r, p)

    h64 = base64.b64encode(hash)
    s64 = base64.b64encode(salt)

    t = 1
    while 2**t < N:
        t += 1
    params = p + (r << 8) + (t << 16)

    return (
        SCRYPT_MCF_ID +
        ('$%06x' % params).encode() +
        b'$' + s64 +
        b'$' + h64
    )


def scrypt_mcf_check(scrypt, mcf, password):
    """Returns True if the password matches the given MCF hash"""
    if not isinstance(mcf, bytes):
        raise TypeError
    if not isinstance(password, bytes):
        raise TypeError

    s = mcf.split(b'$')
    if not (mcf.startswith(SCRYPT_MCF_ID) and len(s) == 5):
        raise ValueError('Unrecognized MCF hash')

    params, s64, h64 = s[2:]
    params = base64.b16decode(params, True)
    salt = base64.b64decode(s64)
    hash = base64.b64decode(h64)

    if len(params) != 3:
        raise ValueError('Unrecognized MCF parameters')
    t, r, p = struct.unpack('3B', params)
    N = 2 ** t

    h = scrypt(password, salt, N=N, r=r, p=p)

    return hash == h




#!/usr/bin/env python

# Automatically generated file, see inline.py

# Copyright (c) 2014 Richard Moore
# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Python implementation of Scrypt password-based key derivation function"""

# Scrypt definition:
# http://www.tarsnap.com/scrypt/scrypt.pdf

# It was originally written for a pure-Python Litecoin CPU miner:
# https://github.com/ricmoo/nightminer
# Imported to this project from:
# https://github.com/ricmoo/pyscrypt
# And owes thanks to:
# https://github.com/wg/scrypt


import hashlib, hmac
import struct


import mcf as mcf_mod

from consts import *


# Python 3.4+ have PBKDF2 in hashlib, so use it...
if 'pbkdf2_hmac' in dir(hashlib):
    _pbkdf2 = hashlib.pbkdf2_hmac
else:
    # but fall back to Python implementation in < 3.4
    from pbkdf2 import pbkdf2_hmac as _pbkdf2


def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
    """Derives a 64-byte hash using the scrypt key-derivarion function

    N must be a power of two larger than 1 but no larger than 2 ** 63 (insane)
    r and p must be positive numbers such that r * p < 2 ** 30

    The default values are:
    N -- 2**14 (~16k)
    r -- 8
    p -- 1

    Memory usage is proportional to N*r. Defaults require about 16 MiB.
    Time taken is proportional to N*p. Defaults take <100ms of a recent x86.

    The last one differs from libscrypt defaults, but matches the 'interactive'
    work factor from the original paper. For long term storage where runtime of
    key derivation is not a problem, you could use 16 as in libscrypt or better
    yet increase N if memory is plentiful.
    """
    def blockxor(source, s_start, dest, d_start, length):
        for i in xrange(length):
            dest[d_start + i] ^= source[s_start + i]


    def integerify(B, r):
        '''"A bijection from ({0, 1} ** k) to {0, ..., (2 ** k) - 1"'''

        Bi = (2 * r - 1) * 16
        return B[Bi]


    def salsa20_8(B, x, src, s_start, dest, d_start):
        '''Salsa20/8 http://en.wikipedia.org/wiki/Salsa20'''

        # Merged blockxor for speed
        for i in xrange(16):
            x[i] = B[i] = B[i] ^ src[s_start + i]

        # This is the actual Salsa 20/8: four identical double rounds
        for i in xrange(4):
            a = (x[0]+x[12]) & 0xffffffff
            b = (x[5]+x[1]) & 0xffffffff
            x[4] ^= (a << 7) | (a >> 25)
            x[9] ^= (b << 7) | (b >> 25)
            a = (x[10]+x[6]) & 0xffffffff
            b = (x[15]+x[11]) & 0xffffffff
            x[14] ^= (a << 7) | (a >> 25)
            x[3] ^= (b << 7) | (b >> 25)
            a = (x[4]+x[0]) & 0xffffffff
            b = (x[9]+x[5]) & 0xffffffff
            x[8] ^= (a << 9) | (a >> 23)
            x[13] ^= (b << 9) | (b >> 23)
            a = (x[14]+x[10]) & 0xffffffff
            b = (x[3]+x[15]) & 0xffffffff
            x[2] ^= (a << 9) | (a >> 23)
            x[7] ^= (b << 9) | (b >> 23)
            a = (x[8]+x[4]) & 0xffffffff
            b = (x[13]+x[9]) & 0xffffffff
            x[12] ^= (a << 13) | (a >> 19)
            x[1] ^= (b << 13) | (b >> 19)
            a = (x[2]+x[14]) & 0xffffffff
            b = (x[7]+x[3]) & 0xffffffff
            x[6] ^= (a << 13) | (a >> 19)
            x[11] ^= (b << 13) | (b >> 19)
            a = (x[12]+x[8]) & 0xffffffff
            b = (x[1]+x[13]) & 0xffffffff
            x[0] ^= (a << 18) | (a >> 14)
            x[5] ^= (b << 18) | (b >> 14)
            a = (x[6]+x[2]) & 0xffffffff
            b = (x[11]+x[7]) & 0xffffffff
            x[10] ^= (a << 18) | (a >> 14)
            x[15] ^= (b << 18) | (b >> 14)
            a = (x[0]+x[3]) & 0xffffffff
            b = (x[5]+x[4]) & 0xffffffff
            x[1] ^= (a << 7) | (a >> 25)
            x[6] ^= (b << 7) | (b >> 25)
            a = (x[10]+x[9]) & 0xffffffff
            b = (x[15]+x[14]) & 0xffffffff
            x[11] ^= (a << 7) | (a >> 25)
            x[12] ^= (b << 7) | (b >> 25)
            a = (x[1]+x[0]) & 0xffffffff
            b = (x[6]+x[5]) & 0xffffffff
            x[2] ^= (a << 9) | (a >> 23)
            x[7] ^= (b << 9) | (b >> 23)
            a = (x[11]+x[10]) & 0xffffffff
            b = (x[12]+x[15]) & 0xffffffff
            x[8] ^= (a << 9) | (a >> 23)
            x[13] ^= (b << 9) | (b >> 23)
            a = (x[2]+x[1]) & 0xffffffff
            b = (x[7]+x[6]) & 0xffffffff
            x[3] ^= (a << 13) | (a >> 19)
            x[4] ^= (b << 13) | (b >> 19)
            a = (x[8]+x[11]) & 0xffffffff
            b = (x[13]+x[12]) & 0xffffffff
            x[9] ^= (a << 13) | (a >> 19)
            x[14] ^= (b << 13) | (b >> 19)
            a = (x[3]+x[2]) & 0xffffffff
            b = (x[4]+x[7]) & 0xffffffff
            x[0] ^= (a << 18) | (a >> 14)
            x[5] ^= (b << 18) | (b >> 14)
            a = (x[9]+x[8]) & 0xffffffff
            b = (x[14]+x[13]) & 0xffffffff
            x[10] ^= (a << 18) | (a >> 14)
            x[15] ^= (b << 18) | (b >> 14)

        # While we are handling the data, write it to the correct dest.
        # The latter half is still part of salsa20
        for i in xrange(16):
            dest[d_start + i] = B[i] = (x[i] + B[i]) & 0xffffffff


    def blockmix_salsa8(BY, Yi, r):
        '''Blockmix; Used by SMix'''

        start = (2 * r - 1) * 16
        X = BY[start:start+16]                             # BlockMix - 1
        tmp = [0]*16

        for i in xrange(2 * r):                            # BlockMix - 2
            #blockxor(BY, i * 16, X, 0, 16)                # BlockMix - 3(inner)
            salsa20_8(X, tmp, BY, i * 16, BY, Yi + i*16)   # BlockMix - 3(outer)
            #array_overwrite(X, 0, BY, Yi + (i * 16), 16)  # BlockMix - 4

        for i in xrange(r):                                # BlockMix - 6
            BY[i * 16:(i * 16)+(16)] = BY[Yi + (i * 2) * 16:(Yi + (i * 2) * 16)+(16)]
            BY[(i + r) * 16:((i + r) * 16)+(16)] = BY[Yi + (i*2 + 1) * 16:(Yi + (i*2 + 1) * 16)+(16)]


    def smix(B, Bi, r, N, V, X):
        '''SMix; a specific case of ROMix based on Salsa20/8'''

        X[0:(0)+(32 * r)] = B[Bi:(Bi)+(32 * r)]

        for i in xrange(N):                                # ROMix - 2
            V[i * (32 * r):(i * (32 * r))+(32 * r)] = X[0:(0)+(32 * r)]
            blockmix_salsa8(X, 32 * r, r)                  # ROMix - 4

        for i in xrange(N):                                # ROMix - 6
            j = integerify(X, r) & (N - 1)                 # ROMix - 7
            blockxor(V, j * (32 * r), X, 0, 32 * r)        # ROMix - 8(inner)
            blockmix_salsa8(X, 32 * r, r)                  # ROMix - 9(outer)

        B[Bi:(Bi)+(32 * r)] = X[0:(0)+(32 * r)]


    if not isinstance(salt, bytes):
        raise TypeError('scrypt salt must be a byte string')
    if N < 2 or (N & (N - 1)):
        raise ValueError('scrypt N must be a power of 2 greater than 1')
    if N > 2 ** 63:
        raise ValueError('N value cannot be larger than 2**63')
    if r <= 0:
        raise ValueError('scrypt r must be positive')
    if p <= 0:
        raise ValueError('scrypt p must be positive')

    B  = _pbkdf2('sha256', password, salt, 1, p * 128 * r)

    # Everything is lists of 32-bit uints for all but pbkdf2
    B  = list(struct.unpack('<%dI' % (len(B) // 4), B))
    XY = [0] * (64 * r)
    V  = [0] * (32 * r * N)

    for i in xrange(p):
        smix(B, i * 32 * r, r, N, V, XY)

    B = struct.pack('<%dI' % len(B), *B)
    return _pbkdf2('sha256', password, B, 1, olen)


def scrypt_mcf(password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p):
    """Derives a Modular Crypt Format hash using the scrypt KDF

    Parameter space is smaller than for scrypt():
    N must be a power of two larger than 1 but no larger than 2 ** 31
    r and p must be positive numbers between 1 and 255
    Salt must be a byte string 1-16 bytes long.

    If no salt is given, 16 random bytes are generated using os.urandom.
    """
    return mcf_mod.scrypt_mcf(scrypt, password, salt, N, r, p)


def scrypt_mcf_check(mcf, password):
    """Returns True if the password matches the given MCF hash"""
    return mcf_mod.scrypt_mcf_check(scrypt, mcf, password)


if __name__ == "__main__":
    import tests
    print('Testing scrypt...')
    tests.run_tests(scrypt, scrypt_mcf, scrypt_mcf_check)




#!/usr/bin/env python

# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Scrypt implementation that calls into system libscrypt"""


import base64
import ctypes, ctypes.util
import os

from ctypes import c_char_p, c_size_t, c_uint64, c_uint32


from consts import *


_libscrypt_soname = ctypes.util.find_library('scrypt')
if _libscrypt_soname is None:
    raise ImportError('Unable to find libscrypt')

try:
    _libscrypt = ctypes.CDLL(_libscrypt_soname)
except OSError:
    raise ImportError('Unable to load libscrypt: ' + _libscrypt_soname)


try:
    _libscrypt_scrypt = _libscrypt.libscrypt_scrypt
except AttributeError:
    raise ImportError('Incompatible libscrypt: ' + _libscrypt_soname)

_libscrypt_scrypt.argtypes = [
    c_char_p,  # password
    c_size_t,  # password length
    c_char_p,  # salt
    c_size_t,  # salt length
    c_uint64,  # N
    c_uint32,  # r
    c_uint32,  # p
    c_char_p,  # out
    c_size_t,  # out length
]


try:
    _libscrypt_mcf = _libscrypt.libscrypt_mcf
except AttributeError:
    raise ImportError('Incompatible libscrypt: ' + _libscrypt_soname)

_libscrypt_mcf.argtypes = [
    c_uint64,  # N
    c_uint32,  # r
    c_uint32,  # p
    c_char_p,  # salt
    c_char_p,  # hash
    c_char_p,  # out (125+ bytes)
]


try:
    _libscrypt_check = _libscrypt.libscrypt_check
except AttributeError:
    raise ImportError('Incompatible libscrypt: ' + _libscrypt_soname)

_libscrypt_check.argtypes = [
    c_char_p,  # mcf (modified)
    c_char_p,  # hash
]


def scrypt(password, salt, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p, olen=64):
    """Derives a 64-byte hash using the scrypt key-derivarion function

    N must be a power of two larger than 1 but no larger than 2 ** 63 (insane)
    r and p must be positive numbers such that r * p < 2 ** 30

    The default values are:
    N -- 2**14 (~16k)
    r -- 8
    p -- 1

    Memory usage is proportional to N*r. Defaults require about 16 MiB.
    Time taken is proportional to N*p. Defaults take <100ms of a recent x86.

    The last one differs from libscrypt defaults, but matches the 'interactive'
    work factor from the original paper. For long term storage where runtime of
    key derivation is not a problem, you could use 16 as in libscrypt or better
    yet increase N if memory is plentiful.
    """
    if not isinstance(password, bytes):
        raise TypeError
    if not isinstance(salt, bytes):
        raise TypeError
    if N > 2**63:
        raise ValueError('N value cannot be larger than 2**63')

    out = ctypes.create_string_buffer(olen)
    ret = _libscrypt_scrypt(password, len(password), salt, len(salt),
                          N, r, p, out, len(out))
    if ret:
        raise ValueError

    return out.raw


def scrypt_mcf(password, salt=None, N=SCRYPT_N, r=SCRYPT_r, p=SCRYPT_p):
    """Derives a Modular Crypt Format hash using the scrypt KDF

    Parameter space is smaller than for scrypt():
    N must be a power of two larger than 1 but no larger than 2 ** 31
    r and p must be positive numbers between 1 and 255
    Salt must be a byte string 1-16 bytes long.

    If no salt is given, 16 random bytes are generated using os.urandom.
    """
    if salt is None:
        salt = os.urandom(16)
    elif not (1 <= len(salt) <= 16):
        raise ValueError('salt must be 1-16 bytes')
    if r == 0 or p == 0:
        raise ValueError('r and p must be positive')
    if N > 2**31:
        raise ValueError('N > 2**31 not supported')
    hash = scrypt(password, salt, N, r, p)

    h64 = base64.b64encode(hash)
    s64 = base64.b64encode(salt)

    out = ctypes.create_string_buffer(SCRYPT_MCF_LEN)
    ret = _libscrypt_mcf(N, r, p, s64, h64, out)
    if not ret:
        raise ValueError

    out = out.raw.strip(b'\0')
    # XXX: Hack to support old libscrypt (like in Ubuntu 14.04)
    if len(out) == 123:
        out = out + b'='

    return out


def scrypt_mcf_check(mcf, password):
    """Returns True if the password matches the given MCF hash"""
    if not isinstance(mcf, bytes):
        raise TypeError
    if not isinstance(password, bytes):
        raise TypeError

    mcfbuf = ctypes.create_string_buffer(mcf)
    ret = _libscrypt_check(mcfbuf, password)
    if ret < 0:
        raise ValueError

    return bool(ret)


if __name__ == "__main__":
    import tests
    print('Testing scrypt...')
    tests.run_tests(scrypt, scrypt_mcf, scrypt_mcf_check)





"""Constants used by scrypt"""

SCRYPT_MCF_ID = b"$s1"
SCRYPT_MCF_LEN = 125

SCRYPT_N = 1<<14
SCRYPT_r = 8
SCRYPT_p = 1

# The last one differs from libscrypt defaults, but matches the 'interactive'
# work factor from the original paper. For long term storage where runtime of
# key derivation is not a problem, you could use 16 as in libscrypt or better
# yet increase N if memory is plentiful.

xrange = xrange if 'xrange' in globals() else range




#!/usr/bin/env python

# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Tests scrypt and PBKDF2 implementations"""


import base64
import hashlib, hmac


def run_tests(scrypt, scrypt_mcf, scrypt_mcf_check, verbose=False, fast=False):
    test_vectors = (
        (b'password', b'NaCl', 1024, 8, 16,
         b'fdbabe1c9d3472007856e7190d01e9fe7c6ad7cbc8237830e77376634b373162'
         b'2eaf30d92e22a3886ff109279d9830dac727afb94a83ee6d8360cbdfa2cc0640',
         b'$s1$0a0810$TmFDbA==$/bq+HJ00cgB4VucZDQHp/nxq18vII3gw53N2Y0s3MWIu'
         b'rzDZLiKjiG/xCSedmDDaxyevuUqD7m2DYMvfoswGQA=='),
        (b'pleaseletmein', b'SodiumChloride', 16384, 8, 1,
         b'7023bdcb3afd7348461c06cd81fd38ebfda8fbba904f8e3ea9b543f6545da1f2'
         b'd5432955613f0fcf62d49705242a9af9e61e85dc0d651e40dfcf017b45575887',
         b'$s1$0e0801$U29kaXVtQ2hsb3JpZGU=$cCO9yzr9c0hGHAbNgf046/2o+7qQT44+'
         b'qbVD9lRdofLVQylVYT8Pz2LUlwUkKpr55h6F3A1lHkDfzwF7RVdYhw=='),
    )
    i = fails = 0
    if fast:
        test_vectors = (
            (b'password', b'NaCl', 2, 8, 1,
             b'e5ed8edc019edfef2d3ced0896faf9eec6921dcc68125ce81c10d53474ce'
             b'1be545979159700d324e77c68d34c553636a8429c4f3c99b9566466877f9'
             b'dca2b92b',
             b'$s1$010801$TmFDbA==$5e2O3AGe3+8tPO0Ilvr57saSHcxoElzoHBDVNHTO'
             b'G+VFl5FZcA0yTnfGjTTFU2NqhCnE88mblWZGaHf53KK5Kw=='),
            (b'pleaseletmein', b'SodiumChloride', 4, 1, 1,
             b'BB1D77016C543A99FE632C9C43C60180FD05E0CAC8B29374DBD1854569CB'
             b'534F487240CFC069D6A59A35F2FA5C7428B21D9BE9F84315446D5371119E'
             b'016FEDF7',
             b'$s1$020101$U29kaXVtQ2hsb3JpZGU=$ux13AWxUOpn+YyycQ8YBgP0F4MrI'
             b'spN029GFRWnLU09IckDPwGnWpZo18vpcdCiyHZvp+EMVRG1TcRGeAW/t9w=='),
        )
    for pw, s, n, r, p, h, m in test_vectors:
        i += 1
        h2 = scrypt(pw, s, n, r, p)
        if h2 != base64.b16decode(h, True):
            print("Test %d.1 failed!" % i)
            print("  scrypt('%s', '%s', %d, %d, %d)" % (pw, s, n, r, p))
            print("  Expected: %s" % h)
            print("  Got:      %s" % base64.b16encode(h2))
            fails += 1
        elif verbose:
            print("Test %d.1 successful!" % i)
        m2 = scrypt_mcf(pw, s, N=n, p=p, r=r)
        if m != m2:
            print("Test %d.2 failed!" % i)
            print("  scrypt_mcf('%s', '%s', %d, %d, %d)" % (pw, s, n, r, p))
            print("  Expected: %s" % m)
            print("  Got:      %s" % m2)
            fails += 1
        elif verbose:
            print("Test %d.2 successful!" % i)
        if not (scrypt_mcf_check(m, pw) and scrypt_mcf_check(m2, pw)):
            print("Test %d.3 failed!" % i)
            print("  scrypt_mcf('%s', '%s', %d, %d, %d)" % (pw, s, n, r, p))
            print("  Expected: %s" % m)
            print("  Got:      %s" % m2)
            print("  scrypt_mcf_check failed!")
            fails += 1
        elif verbose:
            print("Test %d.3 successful!" % i)
        if scrypt_mcf_check(m, b'X' + pw) or scrypt_mcf_check(m2, b'X' + pw):
            print("Test %d.4 failed!" % i)
            print("  scrypt_mcf_check succeeded with wrong password!")
            fails += 1
        elif verbose:
            print("Test %d.4 successful!" % i)

    i += 1
    try:
        scrypt(u'password', b'salt', N=2)
    except TypeError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Unicode password accepted")
        fails += 1

    i += 1
    try:
        scrypt(b'password', u'salt', N=2)
    except TypeError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Unicode salt accepted")
        fails += 1

    i += 1
    try:
        scrypt(b'password', b'salt', N=-1)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Invalid N value -1 accepted")
        fails += 1

    i += 1
    if (scrypt_mcf(b'password', b'salt', N=2) !=
            scrypt_mcf(b'password', b'salt', N=2)):
        print("Test %d.1 failed!" % i)
        print("  Inconsistent MCF!")
        fails += 1
    elif verbose:
        print("Test %d.1 successful!" % i)
    if scrypt_mcf(b'password', N=2) == scrypt_mcf(b'password', N=2):
        print("Test %d.2 failed!" % i)
        print("  Random salts match!")
        fails += 1
    elif verbose:
        print("Test %d.2 successful!" % i)

    i += 1
    try:
        mcf = scrypt_mcf(b'password', b's'*100, N=2)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        if len(mcf) < 150:
            print("Test %d failed!" % i)
            print("  Long salt truncated by scrypt_mcf")
            fails += 1
        elif not scrypt_mcf_check(mcf, b'password'):
            print("Test %d failed!" % i)
            print("  scrypt_mcf[_check] failed with long salt")
        elif scrypt_mcf_check(mcf, b'xpassword'):
            print("Test %d failed!" % i)
            print("  scrypt_mcf[_check] failed with long salt")
            print("  scrypt_mcf_check succeeded with wrong password!")
        elif verbose:
            print("Test %d successful!" % i)

    i += 1
    try:
        scrypt_mcf_check(42, b'password')
    except TypeError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Non-string MCF accepted")
        fails += 1

    i += 1
    try:
        scrypt_mcf_check(b'mcf', 42)
    except TypeError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Non-string password accepted")
        fails += 1

    i += 1
    try:
        scrypt_mcf_check(b'mcf', b'password')
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Invalid MCF not reported")
        fails += 1

    i += 1
    try:
        scrypt_mcf(b'password', b'NaCl', N=2, r=256)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  scrypt_mcf accepted invalid r")
        fails += 1

    i += 1
    try:
        scrypt_mcf(b'password', b'NaCl', N=2, p=256)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  scrypt_mcf accepted invalid p")
        fails += 1

    i += 1
    try:
        scrypt_mcf_check(b'$s1$ffffffff$aaaa$bbbb', b'password')
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  scrypt_mcf_check accepted invalid MCF")
        fails += 1

    i += 1
    if len(scrypt(b'pass', b'salt', N=2, olen=42)) != 42:
        print("Test %d failed!" % i)
        print("  scrypt didn't support irregular length 42")
    elif verbose:
        print("Test %d successful!" % i)

    i += 1
    try:
        scrypt_mcf(b'password', b'NaCl', N=2**42)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    except MemoryError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  N == 2**42 accepted by scrypt_mcf")
        fails += 1

    i += 1
    try:
        scrypt(b'password', b'NaCl', N=2**66+2)
    except ValueError:
        if verbose:
            print("Test %d succeeded!" % i)
    else:
        print("Test %d failed!" % i)
        print("  N == 2**66 + 2 interpreted as 2")
        fails += 1

    i += 1
    try:
        scrypt(b'password', b'NaCl', N=2**66)
    except ValueError:
        if verbose:
            print("Test %d succeeded!" % i)
    else:
        print("Test %d failed!" % i)
        print("  N == 2**66 accepted")
        fails += 1

    i += 1
    try:
        scrypt_mcf(b'password', b'NaCl', N=2, r=0)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Invalid r accepted")
        fails += 1

    i += 1
    try:
        scrypt_mcf(b'password', b'NaCl', N=2, p=0)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Invalid p accepted")
        fails += 1

    i += 1
    try:
        scrypt_mcf(b'password', b'', N=2, r=1, p=1)
    except ValueError:
        if verbose:
            print("Test %d successful!" % i)
    else:
        print("Test %d failed!" % i)
        print("  Empty salt accepted!")
        fails += 1

    if fails:
        print("%d tests failed!" % fails)
    else:
        print("All tests successful!")


def run_tests_pbkdf2(f, verbose=False):
    test_vectors = (
        ('sha1', b'password', b'salt', 1, 20,
         base64.b16decode(b'0c60c80f961f0e71f3a9b524af6012062fe037a6', True)),
        ('sha1', b'pass\0word', b'sa\0lt', 4096, 16,
         base64.b16decode(b'56fa6aa75548099dcc37d7f03425e0c3', True)),
        ('sha256', b'password', b'NaCl', 7, 42,
         base64.b16decode(b'8cb94b8721e20e643be099f3c31d332456b4c'
         b'26f55b6403950267dc2b3c0806bda709a3f2d7f6107db73', True)),
    )
    fails = 0
    for i, param in enumerate(test_vectors):
        if f(*param[:-1]) != param[-1]:
            print("Test %d failed!" % i)
            print("  PBKDF output mismatch")
            print("  Expected: %s" % param[-1])
            print("  Got: %s" % f(*param[:-1]))
            fails += 1
        elif verbose:
            print("Test %d successful!" % i)

    if fails:
        print("%d tests failed!" % fails)
    else:
        print("All tests successful!")



if __name__ == "__main__":
    try:
        import pylibscrypt as cs
        print('Testing C scrypt...')
        run_tests(cs.scrypt, cs.scrypt_mcf, cs.scrypt_mcf_check, fast=True)
    except ImportError:
        print('C scrypt not tested!')

    try:
        import pyscrypt as ms
        print('Testing scrypt module...')
        run_tests(ms.scrypt, ms.scrypt_mcf, ms.scrypt_mcf_check, fast=True)
    except ImportError:
        print('scrypt module not tested!')

    try:
        import pypyscrypt_inline as ps
        print('Testing pure Python scrypt...')
        run_tests(ps.scrypt, ps.scrypt_mcf, ps.scrypt_mcf_check, fast=True)
    except ImportError:
        print('Pure Python scrypt not tested!')

    try:
        import pbkdf2 as pk
        print('Testing pbkdf2...')
        run_tests_pbkdf2(pk.pbkdf2_hmac)
    except ImportError:
        print('Pure Python PBKDF2 not tested!')

    if 'pbkdf2_hmac' in dir(hashlib):
        print('Testing hashlib pbkdf2...')
        run_tests_pbkdf2(hashlib.pbkdf2_hmac)




#!/usr/bin/env python

# Copyright (c) 2014 Jan Varho
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""PBKDF2 in pure Python, compatible with Python3.4 hashlib.pbkdf2_hmac"""


import hashlib
import hmac
import struct

from consts import *


def pbkdf2_hmac(name, password, salt, rounds, dklen=None):
    """Returns the result of the Password-Based Key Derivation Function 2"""
    h = hmac.new(key=password, digestmod=lambda:hashlib.new(name))
    hs = h.copy()
    hs.update(salt)

    blocks = bytearray()
    dklen = hs.digest_size if dklen is None else dklen
    block_count, last_size = divmod(dklen, hs.digest_size)
    block_count += last_size > 0

    for block_number in xrange(1, block_count + 1):
        hb = hs.copy()
        hb.update(struct.pack('>L', block_number))
        U = bytearray(hb.digest())

        if rounds > 1:
            Ui = U
            for i in xrange(rounds - 1):
                hi = h.copy()
                hi.update(Ui)
                Ui = bytearray(hi.digest())
                for j in xrange(hs.digest_size):
                    U[j] ^= Ui[j]

        blocks.extend(U)

    if last_size:
        del blocks[dklen:]
    return bytes(blocks)

